/* plugin_prefs.c
 *
 * Copyright (C) 2000 by Jason Day
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef NEED_PLUGIN_PREFS

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include "prefs.h"

/* These defines copied from libplugin.c */
#define LOG_DEBUG  1    /*debugging info for programers, and bug reports */
#define LOG_INFO   2    /*info, and misc messages */
#define LOG_WARN   4    /*worse messages */
#define LOG_FATAL  8    /*even worse messages */
#define LOG_STDOUT 256  /*messages always go to stdout */
#define LOG_FILE   512  /*messages always go to the log file */
#define LOG_GUI    1024 /*messages always go to the gui window */

static FILE *open_file(char *filename, char *mode);


/* This function copied from prefs.c */
static FILE *open_file(char *filename, char *mode)
{
    char fullname[256];
    FILE *pc_in;

    get_home_file_name(filename, fullname, 255);

    pc_in = fopen(fullname, mode);

    return pc_in;
}


/*
 * WARNING
 * Caller must ensure that which is not out of range!
 */
int jp_get_pref (prefType prefs[], int which, long *n, const char **ret)
{
    if (which < 0) {
        return -1;
    }
    *n = prefs[which].ivalue;
    if (prefs[which].usertype == CHARTYPE) {
        if (ret != NULL) {
            *ret = prefs[which].svalue;
        }
    }
    else {
        if (ret !=NULL) {
            *ret = NULL;
        }
    }
    return 0;
}

/*
 * WARNING
 * Caller must ensure that which is not out of range!
 */
int jp_set_pref (prefType prefs[], int which, long n, const char *string)
{
    if (which < 0) {
        return -1;
    }
    prefs[which].ivalue = n;
    if (string == NULL) {
        prefs[which].svalue[0] = '\0';
        return 0;
    }
    if (prefs[which].filetype == CHARTYPE) {
        strncpy (prefs[which].svalue, string, MAX_PREF_VALUE);
        prefs[which].svalue[MAX_PREF_VALUE - 1] = '\0';
    }
    return 0;
}

/*
 * WARNING
 * Caller must ensure that which is not out of range!
 */
int jp_set_pref_int (prefType prefs[], int which, long n)
{
    if (which < 0) {
        return -1;
    }
    prefs[which].ivalue = n;
    /*
    if (prefs[which]->usertype == CHARTYPE) {
        get_pref_possibility(which, glob_prefs[which].ivalue, glob_prefs[which].svalue);
    }
    */
    return 0;
}

/*
 * WARNING
 * Caller must ensure that which is not out of range!
 */
int jp_set_pref_char (prefType prefs[], int which, char *string)
{
    if (which < 0) {
        return -1;
    }
    if (string == NULL) {
        prefs[which].svalue[0] = '\0';
        return 0;
    }
    if (prefs[which].filetype == CHARTYPE) {
        strncpy (prefs[which].svalue, string, MAX_PREF_VALUE);
        prefs[which].svalue[MAX_PREF_VALUE - 1] = '\0';
    }
    return 0;
}


int jp_read_rc_file (char *filename, prefType prefs[], int num_prefs)
{
    int i;
    FILE *in;
    char line[256];
    char *field1, *field2;
    char *pc;

    in = open_file (filename, "r");
    if (!in) {
        return -1;
    }

    while (!feof (in)) {
        fgets (line, 255, in);
        line[254] = ' ';
        line[255] = '\0';
        field1 = strtok (line, " ");
        field2 = (field1 != NULL) ? strtok (NULL, "\n") : NULL;/* jonh */
        if ((field1 == NULL) || (field2 == NULL)) {
            continue;
        }
        if ((pc = (char *)index (field2, '\n'))) {
            pc[0] = '\0';
        }
        for (i = 0; i < num_prefs; i++) {
            if (!strcmp (prefs[i].name, field1)) {
                if (prefs[i].filetype == INTTYPE) {
                    prefs[i].ivalue = atoi (field2);
                }
                if (prefs[i].filetype == CHARTYPE) {
                    strncpy (prefs[i].svalue, field2, MAX_PREF_VALUE);
                    prefs[i].svalue[MAX_PREF_VALUE - 1] = '\0';
                }
            }
        }
    }
    fclose (in);

    return 0;
}

int jp_write_rc_file (char *filename, prefType prefs[], int num_prefs)
{
    int i;
    FILE *out;

    out = open_file (filename, "w" );
    if (!out) {
        return -1;
    }

    for (i = 0; i < num_prefs; i++) {

        if (prefs[i].filetype == INTTYPE) {
            fprintf (out, "%s %ld\n", prefs[i].name, prefs[i].ivalue);
        }

        if (prefs[i].filetype == CHARTYPE) {
            fprintf (out, "%s %s\n", prefs[i].name, prefs[i].svalue);
        }
    }
    fclose (out);

    return 0;
}

#endif /* NEED_PLUGIN_PREFS */
